"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var Axios_1 = require("./core/Axios");
var util_1 = require("./helpers/util");
var default_1 = require("./default");
var mergeConfig_1 = require("./core/mergeConfig");
var CancelToken_1 = require("./cancel/CancelToken");
var Cancel_1 = require("./cancel/Cancel");
// instance本身是一个函数，但是又拥有Axios类上的所有原型和实例属性
function createInstance(config) {
    var context = new Axios_1.default(config);
    // instance 指向 Axios.prototype.request 方法，并绑定了上下文 context，这样能拥有Axios类上的原型方法
    var instance = Axios_1.default.prototype.request.bind(context);
    (0, util_1.extend)(instance, context);
    return instance;
}
var axios = createInstance(default_1.default);
// 取消请求
axios.CancelToken = CancelToken_1.default;
axios.Cancel = Cancel_1.Cancel;
axios.isCancel = Cancel_1.isCancel;
// axios.create静态方法
// 内部调用了 createInstance 函数，并且把参数 config 与 defaults 合并，作为新的默认配置。注意这里需要 createInstance 函数的返回值类型为 AxiosStatic。
axios.create = function create(config) {
    return createInstance((0, mergeConfig_1.default)(default_1.default, config));
};
// axios.all方法与Promise.all方法是一模一样的，不管是使用方式还是传入的参数都是一模一样的
axios.all = function all(promises) {
    return Promise.all(promises);
};
// axios.all 就是 Promise.all 的封装，它返回的是一个 Promise 数组，then 函数的参数本应是一个参数为 Promise resolves（数组）的函数，在这里使用了 axios.spread 方法。所以 axios.spread 方法是接收一个函数，返回一个新的函数，新函数的结构满足 then 函数的参数结构
axios.spread = function spread(callback) {
    return function wrap(arr) {
        return callback.apply(null, arr);
    };
};
axios.Axios = Axios_1.default;
exports.default = axios;
//# sourceMappingURL=axios.js.map