"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var Cancel_1 = require("./Cancel");
// 在 CancelToken 构造函数内部，实例化一个 pending 状态的 Promise 对象，然后用一个 resolvePromise 变量指向 resolve 函数。调用resolvePromise相当于调用resolve函数
var CancelToken = /** @class */ (function () {
    // 接受一个参数executor 这个参数类型是函数
    function CancelToken(executor) {
        var _this = this;
        var resolvePromise;
        this.promise = new Promise(function (resolve) {
            resolvePromise = resolve;
        });
        // 执行 executor 函数，该函数接收的参数是: cancel 函数
        // cancel 函数就是将来的请求取消触发函数，当外部调用了 cancel 函数，在 cancel 函数内部，会调用 resolvePromise 把 Promise 对象从 pending 状态变为 resolved 状态。
        executor(function (message) {
            if (_this.reason) {
                return;
            }
            _this.reason = new Cancel_1.Cancel(message);
            resolvePromise(_this.reason);
        });
    }
    // 如当一个请求携带的 cancelToken 已经被使用过，那么我们甚至都可以不发送这个请求，只需要抛一个异常即可，并且抛异常的信息就是我们取消的原因
    CancelToken.prototype.throwIfRequested = function () {
        if (this.reason) {
            throw this.reason;
        }
    };
    // source方法返回的是一个对象，里面包含两个属性，分别是：CancelToken类的实例对象token，类型是CancelToken和触发函数cancel，类型是Canceler
    CancelToken.source = function () {
        var cancel;
        var token = new CancelToken(function (c) {
            cancel = c;
        });
        return {
            cancel: cancel,
            token: token
        };
    };
    return CancelToken;
}());
exports.default = CancelToken;
//# sourceMappingURL=CancelToken.js.map