"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
// 定义了一个 InterceptorManager 泛型类，内部维护一个私有属性interceptors
// 该类还提供了三个方法use/forEach/eject
var InterceptorManager = /** @class */ (function () {
    function InterceptorManager() {
        // 在构造器中对interceptors进行初始化
        this.interceptors = [];
    }
    // use实际上是添加拦截器
    InterceptorManager.prototype.use = function (resolved, rejected) {
        this.interceptors.push({
            resolved: resolved,
            rejected: rejected
        });
        // 拦截器的id 此时等于interceptors数组的长度
        return this.interceptors.length - 1;
    };
    // forEach遍历拦截器
    // 把每个拦截器作为传入的fn的参数执行一遍
    InterceptorManager.prototype.forEach = function (fn) {
        this.interceptors.forEach(function (interceptor) {
            if (interceptor !== null) {
                fn(interceptor);
            }
        });
    };
    // eject删除拦截器
    // 删除的时候不能改变数组的长度，所以直接设置为null
    InterceptorManager.prototype.eject = function (id) {
        if (this.interceptors[id]) {
            this.interceptors[id] = null;
        }
    };
    return InterceptorManager;
}());
exports.default = InterceptorManager;
//# sourceMappingURL=InterceptorManager.js.map