"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.transformUrl = void 0;
var url_1 = require("../helpers/url");
var xhr_1 = require("./xhr");
var headers_1 = require("../helpers/headers");
var transform_1 = require("./transform");
function dispatchRequest(config) {
    throwIfCancellationRequested(config);
    processConfig(config);
    // 除了对正常情况的响应数据做转换，我们也需要对异常情况的响应数据做转换
    return (0, xhr_1.default)(config).then(function (res) {
        return transformResponseData(res);
    }, function (e) {
        if (e && e.response) {
            e.response = transformResponseData(e.response);
        }
        return Promise.reject(e);
    });
}
function processConfig(config) {
    config.url = transformUrl(config);
    config.headers = transformHeaders(config);
    // 对请求数据的处理和对响应数据的处理改成使用 transform 函数实现，并把配置中的 transformRequest 及 transformResponse 分别传入
    config.data = (0, transform_1.default)(config.data, config.headers, config.transformRequest);
    config.headers = (0, headers_1.flattenHeaders)(config.headers, config.method);
}
// 发送请求前检查一下配置的 cancelToken 是否已经使用过了，如果已经被用过则不用再次发出请求，直接抛异常。
function throwIfCancellationRequested(config) {
    if (config.cancelToken) {
        config.cancelToken.throwIfRequested();
    }
}
// 先判断传入的url是否是绝对地址，如果不是，则将baseURL与传入的url进行拼接；拼接好之后，将拼接后的url作为请求真正的url发送请求
function transformUrl(config) {
    var url = config.url, params = config.params, paramsSerializer = config.paramsSerializer, baseURL = config.baseURL;
    if (baseURL && !(0, url_1.isAbsoluteURL)(url)) {
        url = (0, url_1.combineURL)(baseURL, url);
    }
    return (0, url_1.buildURL)(url, params, paramsSerializer);
}
exports.transformUrl = transformUrl;
function transformHeaders(config) {
    var _a = config.headers, headers = _a === void 0 ? {} : _a, data = config.data;
    return (0, headers_1.processHeaders)(headers, data);
}
function transformResponseData(res) {
    // 对请求数据的处理和对响应数据的处理改成使用 transform 函数实现，并把配置中的 transformRequest 及 transformResponse 分别传入。
    res.data = (0, transform_1.default)(res.data, res.headers, res.config.transformResponse);
    return res;
}
exports.default = dispatchRequest;
//# sourceMappingURL=dispatchRequest.js.map