"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var util_1 = require("../helpers/util");
var strats = Object.create(null);
// 合并策略 3种策略
// 1.优先获取配置2相应字段的值，如果配置2相应字段没有值，则用配置1相应字段的值
function defaultStrat(val1, val2) {
    return typeof val2 !== 'undefined' ? val2 : val1;
}
// 2.只获取配置2相应字段的值
function fromVal2Strat(val1, val2) {
    if (typeof val2 !== 'undefined') {
        return val2;
    }
}
// 3.深度合并策略，例如headers、auth字段
function deepMergeStrat(val1, val2) {
    if ((0, util_1.isPlainObject)(val2)) {
        // 情况1：val2存在并且是一个普通对象
        return (0, util_1.deepMerge)(val1, val2);
    }
    else if (typeof val2 !== 'undefined') {
        // 情况2:val2存在但不是普通对象
        return val2;
    }
    else if ((0, util_1.isPlainObject)(val1)) {
        // 情况3：val2不存在并且val1是一个普通对象
        return (0, util_1.deepMerge)(val1);
    }
    else {
        // 情况4：val2不存在并且val1不是一个普通对象
        return val1;
    }
}
var stratKeysFromVal2 = ['url', 'params', 'data'];
stratKeysFromVal2.forEach(function (key) {
    strats[key] = fromVal2Strat; // 上述字段的合并策略直接指向fromVal2Strat策略
});
var stratKeysDeepMerge = ['headers', 'auth'];
stratKeysDeepMerge.forEach(function (key) {
    strats[key] = deepMergeStrat;
});
function mergeConfig(config1, config2) {
    if (!config2) {
        config2 = {};
    }
    var config = Object.create(null);
    // 先对config2进行合并
    for (var key in config2) {
        mergeFiled(key);
    }
    // 再对config1进行合并
    for (var key in config1) {
        // 判断字段没有在config2出现过
        if (!config2[key]) {
            mergeFiled(key);
        }
    }
    function mergeFiled(key) {
        // 不同字段对应不同的合并策略函数
        var strat = strats[key] || defaultStrat;
        config[key] = strat(config1[key], config2[key]);
    }
    return config;
}
exports.default = mergeConfig;
//# sourceMappingURL=mergeConfig.js.map