"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var headers_1 = require("./helpers/headers");
var data_1 = require("./helpers/data");
// common 表示对于任何类型的请求都要添加该属性，而 method 表示只有该类型请求方法才会添加对应的属性。
var defaultConfig = {
    method: 'get',
    timeout: 0,
    headers: {
        common: {
            // Accept表示客户端（浏览器）希望接受的数据类型
            Accept: 'application/json, text/plain, */*'
        }
    },
    // 默认配置添加了 transformRequest 和 transformResponse 两个字段
    // 它们的值是一个函数数组或者是一个函数
    transformRequest: [
        function (data, headers) {
            (0, headers_1.processHeaders)(headers, data);
            return (0, data_1.transformRequestData)(data);
        }
    ],
    transformResponse: [
        function (data) {
            return (0, data_1.transformResponseData)(data);
        }
    ],
    xsrfCookieName: 'XSRF-TOKEN',
    xsrfHeaderName: 'X-XSRF-TOKEN',
    validateStatus: function (status) {
        return status >= 200 && status < 300;
    }
};
var methodWithoutData = ['delete', 'get', 'head', 'options'];
methodWithoutData.forEach(function (method) {
    defaultConfig.headers[method] = {};
});
var methodWithData = ['post', 'put', 'patch'];
methodWithData.forEach(function (method) {
    defaultConfig.headers[method] = {
        'Content-Type': 'application/x-www-form-urlencoded'
    };
});
exports.default = defaultConfig;
//# sourceMappingURL=default.js.map