"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.flattenHeaders = exports.parseHeaders = exports.processHeaders = void 0;
var util_1 = require("./util");
function normalizeHeaderName(headers, normalizeName) {
    if (!headers) {
        return;
    }
    Object.keys(headers).forEach(function (name) {
        // 如果传入的是content-type 标准的是应该区分大小写Content-Type
        // 因此把content-type对应的值赋值给Content-Type，然后删除content-type这个字段
        if (name !== normalizeName && name.toUpperCase() === normalizeName.toUpperCase()) {
            headers[normalizeName] = headers[name];
            delete headers[name];
        }
    });
}
function processHeaders(headers, data) {
    normalizeHeaderName(headers, 'Content-Type');
    // 如果数据是普通对象，需要设置Content-Type=application/json;charset=utf-8，才能传送否则服务器无法正确解析客户端传递过来的数据
    if ((0, util_1.isPlainObject)(data)) {
        // 如果用户没有设置headers，提供一个默认设置
        if (headers && !headers['Content-Type']) {
            headers['Content-Type'] = 'application/json;charset=utf-8';
        }
    }
    return headers;
}
exports.processHeaders = processHeaders;
// 对响应头进行处理，解析成一个对象结构
function parseHeaders(headers) {
    // 创造一个对象，用于保存key,value
    var parsed = Object.create(null);
    // 如果没有headers，则返回一个空对象
    if (!headers) {
        return parsed;
    }
    headers.split('\r\n').forEach(function (line) {
        var _a = line.split(':'), key = _a[0], vals = _a.slice(1);
        key = key.trim().toLowerCase();
        if (!key) {
            return;
        }
        var val = vals.join(':').trim();
        parsed[key] = val;
    });
    return parsed;
}
exports.parseHeaders = parseHeaders;
// 合并用户传入的配置和默认配置之后，对headers进行压缩，压缩成一级
function flattenHeaders(headers, method) {
    if (!headers) {
        return headers;
    }
    headers = (0, util_1.deepMerge)(headers.common || {}, headers[method] || {}, headers);
    var methodsToDelete = ['delete', 'get', 'head', 'options', 'post', 'put', 'patch', 'common'];
    // 删掉headers中不需要的字段 比如headers[method]
    methodsToDelete.forEach(function (method) {
        delete headers[method];
    });
    return headers;
}
exports.flattenHeaders = flattenHeaders;
//# sourceMappingURL=headers.js.map