"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.isURLSearchParams = exports.isFormData = exports.deepMerge = exports.extend = exports.isPlainObject = exports.isObject = exports.isDate = void 0;
var toString = Object.prototype.toString;
// val is Date 是自定义的类型保护：类型谓词，会在运行时检查以确保在某个作用域里的类型
// 告诉 TS 编译器，如果上述逻辑语句的返回结果是 true，那么当前判断的 vehicle 变量值的类型是 Car 类型。
function isDate(val) {
    return toString.call(val) === '[object Date]';
}
exports.isDate = isDate;
function isObject(val) {
    return val !== null && typeof val === 'object';
}
exports.isObject = isObject;
function isPlainObject(val) {
    return toString.call(val) === '[object Object]';
}
exports.isPlainObject = isPlainObject;
// extend 的最终目的是把 from 里的属性都扩展到 to 中，包括原型上的属性。
function extend(to, from) {
    // 遍历from上的所有属性
    for (var key in from) {
        ;
        to[key] = from[key];
    }
    return to;
}
exports.extend = extend;
// 普通对象的深拷贝和合并，支持传入多个参数
// 但是深拷贝的对象result[key] args[0].key会被args[1].key结果覆盖掉
function deepMerge() {
    var args = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        args[_i] = arguments[_i];
    }
    var result = Object.create(null);
    args.forEach(function (arg) {
        if (arg) {
            // Object.keys只遍历自身上的属性或者方法
            Object.keys(arg).forEach(function (key) {
                var val = arg[key];
                if (isPlainObject(val)) {
                    // 情况1：如果val是对象，进行递归
                    if (isPlainObject(result[key])) {
                        // 递归之前，判断result[key]是否为对象再进行赋值（因为支持多个参数传递）
                        result[key] = deepMerge(result[key], val);
                    }
                    else {
                        result[key] = deepMerge({}, val);
                    }
                }
                else {
                    // 情况2：如果val不是对象
                    result[key] = val;
                }
            });
        }
    });
    return result;
}
exports.deepMerge = deepMerge;
// 如果请求的数据是 FormData 类型，我们应该主动删除请求 headers 中的 Content-Type 字段，让浏览器自动根据请求数据设置 Content-Type。
// 比如当我们通过 FormData 上传文件的时候，浏览器会把请求 headers 中的 Content-Type 设置为 multipart/form-data。
// 添加一个判断 FormData 的方法
function isFormData(val) {
    return typeof val !== 'undefined' && val instanceof FormData;
}
exports.isFormData = isFormData;
// 对 params 类型判断，如果它是一个 URLSearchParams 对象实例的话，我们直接返回它 toString 后的结果
function isURLSearchParams(val) {
    return typeof val !== 'undefined' && val instanceof URLSearchParams;
}
exports.isURLSearchParams = isURLSearchParams;
//# sourceMappingURL=util.js.map