(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory() :
  typeof define === 'function' && define.amd ? define(factory) :
  (global = typeof globalThis !== 'undefined' ? globalThis : global || self, global.tsAxios = factory());
})(this, (function () { 'use strict';

  var toString = Object.prototype.toString;
  // val is Date 是自定义的类型保护：类型谓词，会在运行时检查以确保在某个作用域里的类型
  // 告诉 TS 编译器，如果上述逻辑语句的返回结果是 true，那么当前判断的 vehicle 变量值的类型是 Car 类型。
  function isDate(val) {
      return toString.call(val) === '[object Date]';
  }
  function isObject(val) {
      return val !== null && typeof val === 'object';
  }
  function isPlainObject(val) {
      return toString.call(val) === '[object Object]';
  }
  // extend 的最终目的是把 from 里的属性都扩展到 to 中，包括原型上的属性。
  function extend(to, from) {
      // 遍历from上的所有属性
      for (var key in from) {
          to[key] = from[key];
      }
      return to;
  }
  // 普通对象的深拷贝和合并，支持传入多个参数
  // 但是深拷贝的对象result[key] args[0].key会被args[1].key结果覆盖掉
  function deepMerge() {
      var args = [];
      for (var _i = 0; _i < arguments.length; _i++) {
          args[_i] = arguments[_i];
      }
      var result = Object.create(null);
      args.forEach(function (arg) {
          if (arg) {
              // Object.keys只遍历自身上的属性或者方法
              Object.keys(arg).forEach(function (key) {
                  var val = arg[key];
                  if (isPlainObject(val)) {
                      // 情况1：如果val是对象，进行递归
                      if (isPlainObject(result[key])) {
                          // 递归之前，判断result[key]是否为对象再进行赋值（因为支持多个参数传递）
                          result[key] = deepMerge(result[key], val);
                      }
                      else {
                          result[key] = deepMerge({}, val);
                      }
                  }
                  else {
                      // 情况2：如果val不是对象
                      result[key] = val;
                  }
              });
          }
      });
      return result;
  }
  // 如果请求的数据是 FormData 类型，我们应该主动删除请求 headers 中的 Content-Type 字段，让浏览器自动根据请求数据设置 Content-Type。
  // 比如当我们通过 FormData 上传文件的时候，浏览器会把请求 headers 中的 Content-Type 设置为 multipart/form-data。
  // 添加一个判断 FormData 的方法
  function isFormData(val) {
      return typeof val !== 'undefined' && val instanceof FormData;
  }
  // 对 params 类型判断，如果它是一个 URLSearchParams 对象实例的话，我们直接返回它 toString 后的结果
  function isURLSearchParams(val) {
      return typeof val !== 'undefined' && val instanceof URLSearchParams;
  }

  // encodeURIComponent() 函数可把字符串作为 URI 组件进行编码
  // 该方法不会对 ASCII 字母和数字进行编码，也不会对这些 ASCII 标点符号进行编码： - _ . ! ~ * ' ( )
  // 对于字符 @ : $ , [ ]，我们是允许出现在 url 中的，不希望被 encode
  // 空格字符会被替换成 +
  function encode(val) {
      return encodeURIComponent(val)
          .replace(/%40/g, '@')
          .replace(/%3A/gi, ':')
          .replace(/%24/g, '$')
          .replace(/%2C/gi, ',')
          .replace(/%20/g, '+')
          .replace(/%5B/gi, '[')
          .replace(/%5D/gi, ']');
  }
  function buildURL(url, params, paramsSerializer) {
      // 如果没传params 就不用进行url的拼接
      if (!params) {
          return url;
      }
      var serializeParams;
      if (paramsSerializer) {
          serializeParams = paramsSerializer(params);
      }
      else if (isURLSearchParams(params)) {
          serializeParams = params.toString();
      }
      else {
          var parts_1 = [];
          Object.keys(params).forEach(function (key) {
              var val = params[key];
              if (val === null || typeof val === 'undefined') {
                  return;
              }
              var values = [];
              if (Array.isArray(val)) {
                  values = val;
                  key += '[]'; // 参数值为数组时  拼接后的url为/base/get?foo[]=bar&foo[]=baz ，所以key要+='[]'
              }
              else {
                  values = [val]; // 把val放在数组里，这是为了调用forEach方法
              }
              values.forEach(function (val) {
                  // 遍历values的每一项val
                  if (isDate(val)) {
                      // val如果是日期，调用toISOString进行拼接
                      val = val.toISOString();
                  }
                  else if (isObject(val)) {
                      // val如果是对象，调用JSON.stringify继续拼接
                      val = JSON.stringify(val);
                  }
                  parts_1.push("".concat(encode(key), "=").concat(encode(val)));
              });
          });
          // parts的每一项通过&来连接
          serializeParams = parts_1.join('&');
      }
      if (serializeParams) {
          // 看是否有哈希标识符，有的话要忽略#及#后面的内容
          var markIndex = url.indexOf('#');
          if (markIndex !== -1) {
              url = url.slice(0, markIndex);
          }
          // 如果前面没有? 需要拼上? 否则 拼上 &
          url += (url.indexOf('?') === -1 ? '?' : '&') + serializeParams;
      }
      return url;
  }
  // 如果URL以“<scheme>：//”或“//”（协议相对URL）开头，则该URL被视为绝对值。
  // RFC 3986将方案名称定义为以字母开头的字符序列，后跟字母，数字，加号，句点或连字符的任意组合。
  function isAbsoluteURL(url) {
      return /^([a-z][a-z\d\+\-\.]*:)?\/\//i.test(url);
  }
  function combineURL(baseURL, relativeURL) {
      return relativeURL ? baseURL.replace(/\/+$/, '') + '/' + relativeURL.replace(/^\/+/, '') : baseURL;
  }
  function isURLSameOrigin(requestURL) {
      var urlParsingNode = document.createElement('a');
      // 获取当前页面地址的协议、域名、端口号
      var currentOrigin = resolveURL(window.location.href);
      // 获取请求的url的协议、域名、端口号
      var parsedOrigin = resolveURL(requestURL);
      // 比较是否相同，即判断是否是同源还是跨域
      return (parsedOrigin.protocol === currentOrigin.protocol &&
          parsedOrigin.host === currentOrigin.host &&
          parsedOrigin.port === currentOrigin.port);
      // 创建一个可以通过URL获取协议、域名、端口号的函数
      function resolveURL(url) {
          urlParsingNode.setAttribute('href', url);
          return {
              protocol: urlParsingNode.protocol ? urlParsingNode.protocol.replace(/:$/, '') : '',
              host: urlParsingNode.host,
              port: urlParsingNode.port
          };
      }
  }

  function normalizeHeaderName(headers, normalizeName) {
      if (!headers) {
          return;
      }
      Object.keys(headers).forEach(function (name) {
          // 如果传入的是content-type 标准的是应该区分大小写Content-Type
          // 因此把content-type对应的值赋值给Content-Type，然后删除content-type这个字段
          if (name !== normalizeName && name.toUpperCase() === normalizeName.toUpperCase()) {
              headers[normalizeName] = headers[name];
              delete headers[name];
          }
      });
  }
  function processHeaders(headers, data) {
      normalizeHeaderName(headers, 'Content-Type');
      // 如果数据是普通对象，需要设置Content-Type=application/json;charset=utf-8，才能传送否则服务器无法正确解析客户端传递过来的数据
      if (isPlainObject(data)) {
          // 如果用户没有设置headers，提供一个默认设置
          if (headers && !headers['Content-Type']) {
              headers['Content-Type'] = 'application/json;charset=utf-8';
          }
      }
      return headers;
  }
  // 对响应头进行处理，解析成一个对象结构
  function parseHeaders(headers) {
      // 创造一个对象，用于保存key,value
      var parsed = Object.create(null);
      // 如果没有headers，则返回一个空对象
      if (!headers) {
          return parsed;
      }
      headers.split('\r\n').forEach(function (line) {
          var _a = line.split(':'), key = _a[0], vals = _a.slice(1);
          key = key.trim().toLowerCase();
          if (!key) {
              return;
          }
          var val = vals.join(':').trim();
          parsed[key] = val;
      });
      return parsed;
  }
  // 合并用户传入的配置和默认配置之后，对headers进行压缩，压缩成一级
  function flattenHeaders(headers, method) {
      if (!headers) {
          return headers;
      }
      headers = deepMerge(headers.common || {}, headers[method] || {}, headers);
      var methodsToDelete = ['delete', 'get', 'head', 'options', 'post', 'put', 'patch', 'common'];
      // 删掉headers中不需要的字段 比如headers[method]
      methodsToDelete.forEach(function (method) {
          delete headers[method];
      });
      return headers;
  }

  /*! *****************************************************************************
  Copyright (c) Microsoft Corporation. All rights reserved.
  Licensed under the Apache License, Version 2.0 (the "License"); you may not use
  this file except in compliance with the License. You may obtain a copy of the
  License at http://www.apache.org/licenses/LICENSE-2.0

  THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
  KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
  WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
  MERCHANTABLITY OR NON-INFRINGEMENT.

  See the Apache Version 2.0 License for specific language governing permissions
  and limitations under the License.
  ***************************************************************************** */
  /* global Reflect, Promise */

  var extendStatics = function(d, b) {
      extendStatics = Object.setPrototypeOf ||
          ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
          function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
      return extendStatics(d, b);
  };

  function __extends(d, b) {
      extendStatics(d, b);
      function __() { this.constructor = d; }
      d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
  }

  var AxiosError = /** @class */ (function (_super) {
      __extends(AxiosError, _super);
      function AxiosError(message, config, code, request, response) {
          var _this = 
          // message是父类Error身上的属性
          _super.call(this, message) || this;
          _this.config = config;
          _this.isAxiosError = true;
          _this.code = code;
          _this.request = request;
          _this.response = response;
          // 这是为了解决ts在继承一些内置对象时会出现bug ，无法访问原型链上的属性和方法，需要指定实例对象的原型为AxiosError.prototype
          Object.setPrototypeOf(_this, AxiosError.prototype);
          return _this;
      }
      return AxiosError;
  }(Error));
  // 为了方便使用，对外暴露一个createError的工厂方法
  function createError(message, config, code, request, response) {
      var error = new AxiosError(message, config, code, request, response);
      return error;
  }

  // cookie读取，利用了正则表达式可以解析到 name 对应的值。
  var cookie = {
      read: function (name) {
          var match = document.cookie.match(new RegExp('(^|;\\s*)(' + name + ')=([^;]*)'));
          return match ? decodeURIComponent(match[3]) : null;
      }
  };

  //  整个流程分为 7 步：
  //  创建一个 request 实例。
  //  执行 request.open 方法初始化。
  //  执行 configureRequest 配置 request 对象。
  //  执行 addEvents 给 request 添加事件处理函数。
  //  执行 processHeaders 处理请求 headers。
  //  执行 processCancel 处理请求取消逻辑。
  //  执行 request.send 方法发送请求。
  function xhr(config) {
      // Promise封装一个异步请求，用户可以通过then处理服务端返回的数据
      return new Promise(function (resolve, reject) {
          // config是用户设置的请求信息
          var _a = config.data, data = _a === void 0 ? null : _a, url = config.url, _b = config.method, method = _b === void 0 ? 'get' : _b, headers = config.headers, responseType = config.responseType, timeout = config.timeout, cancelToken = config.cancelToken, withCredentials = config.withCredentials, xsrfCookieName = config.xsrfCookieName, xsrfHeaderName = config.xsrfHeaderName, onDownloadProgress = config.onDownloadProgress, onUploadProgress = config.onUploadProgress, auth = config.auth, validateStatus = config.validateStatus;
          var request = new XMLHttpRequest();
          // true开启异步
          request.open(method.toUpperCase(), url, true);
          // 配置requestConfig对象
          configRequest();
          // 给request添加事件处理函数
          addEvents();
          // 处理headers
          processHeaders();
          // 处理请求取消逻辑
          processCancel();
          request.send(data);
          function configRequest() {
              // 如果用户指定了返回数据的类型
              if (responseType) {
                  request.responseType = responseType;
              }
              // 如果用户指定请求的超时时间，超过一定时间没有响应触发timeout事件
              if (timeout) {
                  request.timeout = timeout;
              }
              // 如果用户设置withCredentials，发送请求时会携带cookie
              if (withCredentials) {
                  request.withCredentials = withCredentials;
              }
          }
          function addEvents() {
              request.onreadystatechange = function handleLoad() {
                  if (request.readyState !== 4) {
                      return;
                  }
                  if (request.status === 0) {
                      return;
                  }
                  // 获取所有响应头
                  var responseHeaders = parseHeaders(request.getAllResponseHeaders());
                  // 获取返回的响应数据
                  var responseData = responseType && responseType !== 'text' ? request.response : request.responseText;
                  var response = {
                      data: responseData,
                      status: request.status,
                      statusText: request.statusText,
                      headers: responseHeaders,
                      config: config,
                      request: request
                  };
                  handleResponse(response);
              };
              // 处理网络错误
              request.onerror = function handleError() {
                  reject(createError('Network Error', config, null, request));
              };
              // 处理超时
              request.ontimeout = function handleTimeout() {
                  reject(createError("Timeout of ".concat(config.timeout, " ms exceeded"), config, 'ECONNABORTED', request));
              };
              // 下载和上传的进度
              if (onDownloadProgress) {
                  request.onprogress = onDownloadProgress;
              }
              if (onUploadProgress) {
                  request.upload.onprogress = onUploadProgress;
              }
          }
          function processHeaders() {
              // 如果请求是个 FormData 类型，则删除 headers['Content-Type'] 让浏览器自动为请求带上合适的 Content-Type
              if (isFormData(data)) {
                  delete headers['Content-Type'];
              }
              // 读取和设置cookie
              if ((withCredentials || isURLSameOrigin(url)) && xsrfCookieName) {
                  var xsrfValue = cookie.read(xsrfCookieName);
                  if (xsrfValue) {
                      headers[xsrfHeaderName] = xsrfValue;
                  }
              }
              // 如果用户设置了auth,如果配置了就把该属性经过base64 加密后添加到请求 headers 中的 Authorization 属性上
              if (auth) {
                  headers['Authorization'] = "Basic ".concat(btoa("".concat(auth.username, " : ").concat(auth.password)));
              }
              // 如果传入的data为空，请求headers配置Content-Type是没有意义的，所以删除掉
              Object.keys(headers).forEach(function (name) {
                  if (data === null && name.toLocaleLowerCase() === 'content-type') {
                      delete headers[name];
                  }
                  else {
                      request.setRequestHeader(name, headers[name]);
                  }
              });
          }
          function processCancel() {
              // 首先判断用户是否配置的cancelToken，如果没有配置，表示没有取消请求这项需求；
              // 如果配置cancelToken，并且当外部调用了请求取消触发函数，此时cancelToken.promise会变成resolved 状态，然后就会执行then函数，在then函数内部调用XMLHttpRequest对象上的abort()取消请求
              if (cancelToken) {
                  cancelToken.promise
                      .then(function (reason) {
                      request.abort();
                      reject(reason);
                  })
                      .catch(function () {
                      // do nothing
                  });
              }
          }
          function handleResponse(response) {
              // 自定义合法状态码
              if (!validateStatus || validateStatus(response.status)) {
                  resolve(response);
              }
              else {
                  reject(createError("Request failed with status code ".concat(response.status), config, null, request, response));
              }
          }
      });
  }

  // transform 函数中接收 data、headers、fns 3 个参数，其中 fns 代表一个或者多个转换函数。
  // 内部逻辑很简单，遍历 fns，执行这些转换函数，并且把 data 和 headers 作为参数传入，每个转换函数返回的 data 会作为下一个转换函数的参数 data 传入。
  function transform(data, headers, fns) {
      if (!fns) {
          return data;
      }
      // 如果不是数组则转为数组，为了统一遍历
      if (!Array.isArray(fns)) {
          fns = [fns];
      }
      fns.forEach(function (fn) {
          data = fn(data, headers);
      });
      return data;
  }

  function dispatchRequest(config) {
      throwIfCancellationRequested(config);
      processConfig(config);
      // 除了对正常情况的响应数据做转换，我们也需要对异常情况的响应数据做转换
      return xhr(config).then(function (res) {
          return transformResponseData$1(res);
      }, function (e) {
          if (e && e.response) {
              e.response = transformResponseData$1(e.response);
          }
          return Promise.reject(e);
      });
  }
  function processConfig(config) {
      config.url = transformUrl(config);
      config.headers = transformHeaders(config);
      // 对请求数据的处理和对响应数据的处理改成使用 transform 函数实现，并把配置中的 transformRequest 及 transformResponse 分别传入
      config.data = transform(config.data, config.headers, config.transformRequest);
      config.headers = flattenHeaders(config.headers, config.method);
  }
  // 发送请求前检查一下配置的 cancelToken 是否已经使用过了，如果已经被用过则不用再次发出请求，直接抛异常。
  function throwIfCancellationRequested(config) {
      if (config.cancelToken) {
          config.cancelToken.throwIfRequested();
      }
  }
  // 先判断传入的url是否是绝对地址，如果不是，则将baseURL与传入的url进行拼接；拼接好之后，将拼接后的url作为请求真正的url发送请求
  function transformUrl(config) {
      var url = config.url, params = config.params, paramsSerializer = config.paramsSerializer, baseURL = config.baseURL;
      if (baseURL && !isAbsoluteURL(url)) {
          url = combineURL(baseURL, url);
      }
      return buildURL(url, params, paramsSerializer);
  }
  function transformHeaders(config) {
      var _a = config.headers, headers = _a === void 0 ? {} : _a, data = config.data;
      return processHeaders(headers, data);
  }
  function transformResponseData$1(res) {
      // 对请求数据的处理和对响应数据的处理改成使用 transform 函数实现，并把配置中的 transformRequest 及 transformResponse 分别传入。
      res.data = transform(res.data, res.headers, res.config.transformResponse);
      return res;
  }

  // 定义了一个 InterceptorManager 泛型类，内部维护一个私有属性interceptors
  // 该类还提供了三个方法use/forEach/eject
  var InterceptorManager = /** @class */ (function () {
      function InterceptorManager() {
          // 在构造器中对interceptors进行初始化
          this.interceptors = [];
      }
      // use实际上是添加拦截器
      InterceptorManager.prototype.use = function (resolved, rejected) {
          this.interceptors.push({
              resolved: resolved,
              rejected: rejected
          });
          // 拦截器的id 此时等于interceptors数组的长度
          return this.interceptors.length - 1;
      };
      // forEach遍历拦截器
      // 把每个拦截器作为传入的fn的参数执行一遍
      InterceptorManager.prototype.forEach = function (fn) {
          this.interceptors.forEach(function (interceptor) {
              if (interceptor !== null) {
                  fn(interceptor);
              }
          });
      };
      // eject删除拦截器
      // 删除的时候不能改变数组的长度，所以直接设置为null
      InterceptorManager.prototype.eject = function (id) {
          if (this.interceptors[id]) {
              this.interceptors[id] = null;
          }
      };
      return InterceptorManager;
  }());

  var strats = Object.create(null);
  // 合并策略 3种策略
  // 1.优先获取配置2相应字段的值，如果配置2相应字段没有值，则用配置1相应字段的值
  function defaultStrat(val1, val2) {
      return typeof val2 !== 'undefined' ? val2 : val1;
  }
  // 2.只获取配置2相应字段的值
  function fromVal2Strat(val1, val2) {
      if (typeof val2 !== 'undefined') {
          return val2;
      }
  }
  // 3.深度合并策略，例如headers、auth字段
  function deepMergeStrat(val1, val2) {
      if (isPlainObject(val2)) {
          // 情况1：val2存在并且是一个普通对象
          return deepMerge(val1, val2);
      }
      else if (typeof val2 !== 'undefined') {
          // 情况2:val2存在但不是普通对象
          return val2;
      }
      else if (isPlainObject(val1)) {
          // 情况3：val2不存在并且val1是一个普通对象
          return deepMerge(val1);
      }
      else {
          // 情况4：val2不存在并且val1不是一个普通对象
          return val1;
      }
  }
  var stratKeysFromVal2 = ['url', 'params', 'data'];
  stratKeysFromVal2.forEach(function (key) {
      strats[key] = fromVal2Strat; // 上述字段的合并策略直接指向fromVal2Strat策略
  });
  var stratKeysDeepMerge = ['headers', 'auth'];
  stratKeysDeepMerge.forEach(function (key) {
      strats[key] = deepMergeStrat;
  });
  function mergeConfig(config1, config2) {
      if (!config2) {
          config2 = {};
      }
      var config = Object.create(null);
      // 先对config2进行合并
      for (var key in config2) {
          mergeFiled(key);
      }
      // 再对config1进行合并
      for (var key in config1) {
          // 判断字段没有在config2出现过
          if (!config2[key]) {
              mergeFiled(key);
          }
      }
      function mergeFiled(key) {
          // 不同字段对应不同的合并策略函数
          var strat = strats[key] || defaultStrat;
          config[key] = strat(config1[key], config2[key]);
      }
      return config;
  }

  // 文件名首字母大写表明它是一个类
  var Axios = /** @class */ (function () {
      function Axios(initConfig) {
          this.defaults = initConfig;
          this.interceptors = {
              // request是对config进行操作，response是对响应信息进行操作
              request: new InterceptorManager(),
              response: new InterceptorManager()
          };
      }
      // request方法 用户可以配置config method包括get/put/post/delete等等
      // 实际上是通过dispatchRequest函数来实现，而dispatchRequest函数是基于xhr函数
      // 支持传入一个或两个参数
      // 虽然修改 request 的实现，支持了 2 种参数，但是我们对外提供的 request 接口仍然不变（Axios类中的request参数类型），可以理解为这仅仅是内部的实现的修改，与对外接口不必一致，只要保留实现兼容接口即可
      // 也就是说当直接调用axios函数时可以传一个(config)或者两个参数(url和config)，但是调用axios.request时就只能传一个参数config
      Axios.prototype.request = function (url, config) {
          // 函数重载
          if (typeof url === 'string') {
              // 如果传递了url 还要判断config是否传入
              if (!config) {
                  config = {};
              }
              config.url = url;
          }
          else {
              // 如果 url 不是字符串类型，则说明我们传入的就是单个参数，且 url 就是 config，因此把 url 赋值给 config
              config = url;
          }
          // 根据用户传入的配置和默认配置进行合并
          config = mergeConfig(this.defaults, config);
          // 将config的方法转化为小写,这么做的目的也是为了之后 flattenHeaders(处理响应请求头的函数) 能正常处理这些 method
          config.method = config.method.toLowerCase();
          // 构造一个 PromiseChain 类型的数组 chain，并把 dispatchRequest 函数赋值给 resolved 属性
          var chain = [
              {
                  resolved: dispatchRequest,
                  rejected: undefined
              }
          ];
          this.interceptors.request.forEach(function (interceptor) {
              // 对于请求拦截器来说，后添加的拦截器先执行，通过unshift先放到数组开头
              chain.unshift(interceptor);
          });
          this.interceptors.response.forEach(function (interceptor) {
              // 对于响应拦截器来说，先添加的拦截器先执行
              chain.push(interceptor);
          });
          // 定义一个已经 resolve 的 promise，循环这个 chain，拿到每个拦截器对象，把它们的 resolved 函数和 rejected 函数添加到 promise.then 的参数中
          // 这样就相当于通过 Promise 的链式调用方式，实现了拦截器一层层的链式调用的效果。
          var promise = Promise.resolve(config);
          while (chain.length) {
              // chain有可能是Promise 也有可能是空，用!断言不为空
              // shift删除数组第一个元素并返回这个元素
              var _a = chain.shift(), resolved = _a.resolved, rejected = _a.rejected;
              promise = promise.then(resolved, rejected);
          }
          // return dispatchRequest(config)
          return promise;
      };
      // 对于 get、delete、head、options、post、patch、put 这些方法，都是对外提供的语法糖，内部都是通过调用 request 方法实现发送请求，只不过在调用之前对 config 做了一层合并处理
      Axios.prototype.get = function (url, config) {
          return this._requestMethodWithoutData('get', url, config);
      };
      Axios.prototype.delete = function (url, config) {
          return this._requestMethodWithoutData('delete', url, config);
      };
      Axios.prototype.head = function (url, config) {
          return this._requestMethodWithoutData('head', url, config);
      };
      Axios.prototype.options = function (url, config) {
          return this._requestMethodWithoutData('options', url, config);
      };
      // post、put、patch的参数多了data
      Axios.prototype.post = function (url, data, config) {
          return this._requestMethodWithData('post', url, config, data);
      };
      Axios.prototype.put = function (url, data, config) {
          return this._requestMethodWithData('put', url, config, data);
      };
      Axios.prototype.patch = function (url, data, config) {
          return this._requestMethodWithData('patch', url, config, data);
      };
      Axios.prototype.getUri = function (config) {
          var finalConfig = mergeConfig(this.defaults, config);
          return transformUrl(finalConfig);
      };
      Axios.prototype._requestMethodWithoutData = function (method, url, config) {
          return this.request(Object.assign(config || {}, {
              method: method,
              url: url
          }));
      };
      Axios.prototype._requestMethodWithData = function (method, url, config, data) {
          return this.request(Object.assign(config || {}, {
              method: method,
              url: url,
              data: data
          }));
      };
      return Axios;
  }());

  // 请求的data如果是普通对象，转换成JSON字符串
  function transformRequestData(data) {
      if (isPlainObject(data)) {
          // 普通对象需要转为JSON字符串才能被写进body被传递
          return JSON.stringify(data);
      }
      return data;
  }
  // 响应的data从JSON字符串转为普通对象
  function transformResponseData(data) {
      if (typeof data === 'string') {
          try {
              data = JSON.parse(data);
          }
          catch (e) {
              // do nothing
          }
      }
      return data;
  }

  // common 表示对于任何类型的请求都要添加该属性，而 method 表示只有该类型请求方法才会添加对应的属性。
  var defaultConfig = {
      method: 'get',
      timeout: 0,
      headers: {
          common: {
              // Accept表示客户端（浏览器）希望接受的数据类型
              Accept: 'application/json, text/plain, */*'
          }
      },
      // 默认配置添加了 transformRequest 和 transformResponse 两个字段
      // 它们的值是一个函数数组或者是一个函数
      transformRequest: [
          function (data, headers) {
              processHeaders(headers, data);
              return transformRequestData(data);
          }
      ],
      transformResponse: [
          function (data) {
              return transformResponseData(data);
          }
      ],
      xsrfCookieName: 'XSRF-TOKEN',
      xsrfHeaderName: 'X-XSRF-TOKEN',
      validateStatus: function (status) {
          return status >= 200 && status < 300;
      }
  };
  var methodWithoutData = ['delete', 'get', 'head', 'options'];
  methodWithoutData.forEach(function (method) {
      defaultConfig.headers[method] = {};
  });
  var methodWithData = ['post', 'put', 'patch'];
  methodWithData.forEach(function (method) {
      defaultConfig.headers[method] = {
          'Content-Type': 'application/x-www-form-urlencoded'
      };
  });

  var Cancel = /** @class */ (function () {
      function Cancel(message) {
          this.message = message;
      }
      return Cancel;
  }());
  // 创建好Cancel类之后，我们还应该创建一个isCancel函数，该函数用来判断异常对象是不是取消原因对象，返回true或false
  function isCancel(value) {
      return value instanceof Cancel;
  }

  // 在 CancelToken 构造函数内部，实例化一个 pending 状态的 Promise 对象，然后用一个 resolvePromise 变量指向 resolve 函数。调用resolvePromise相当于调用resolve函数
  var CancelToken = /** @class */ (function () {
      // 接受一个参数executor 这个参数类型是函数
      function CancelToken(executor) {
          var _this = this;
          var resolvePromise;
          this.promise = new Promise(function (resolve) {
              resolvePromise = resolve;
          });
          // 执行 executor 函数，该函数接收的参数是: cancel 函数
          // cancel 函数就是将来的请求取消触发函数，当外部调用了 cancel 函数，在 cancel 函数内部，会调用 resolvePromise 把 Promise 对象从 pending 状态变为 resolved 状态。
          executor(function (message) {
              if (_this.reason) {
                  return;
              }
              _this.reason = new Cancel(message);
              resolvePromise(_this.reason);
          });
      }
      // 如当一个请求携带的 cancelToken 已经被使用过，那么我们甚至都可以不发送这个请求，只需要抛一个异常即可，并且抛异常的信息就是我们取消的原因
      CancelToken.prototype.throwIfRequested = function () {
          if (this.reason) {
              throw this.reason;
          }
      };
      // source方法返回的是一个对象，里面包含两个属性，分别是：CancelToken类的实例对象token，类型是CancelToken和触发函数cancel，类型是Canceler
      CancelToken.source = function () {
          var cancel;
          var token = new CancelToken(function (c) {
              cancel = c;
          });
          return {
              cancel: cancel,
              token: token
          };
      };
      return CancelToken;
  }());

  // instance本身是一个函数，但是又拥有Axios类上的所有原型和实例属性
  function createInstance(config) {
      var context = new Axios(config);
      // instance 指向 Axios.prototype.request 方法，并绑定了上下文 context，这样能拥有Axios类上的原型方法
      var instance = Axios.prototype.request.bind(context);
      extend(instance, context);
      return instance;
  }
  var axios = createInstance(defaultConfig);
  // 取消请求
  axios.CancelToken = CancelToken;
  axios.Cancel = Cancel;
  axios.isCancel = isCancel;
  // axios.create静态方法
  // 内部调用了 createInstance 函数，并且把参数 config 与 defaults 合并，作为新的默认配置。注意这里需要 createInstance 函数的返回值类型为 AxiosStatic。
  axios.create = function create(config) {
      return createInstance(mergeConfig(defaultConfig, config));
  };
  // axios.all方法与Promise.all方法是一模一样的，不管是使用方式还是传入的参数都是一模一样的
  axios.all = function all(promises) {
      return Promise.all(promises);
  };
  // axios.all 就是 Promise.all 的封装，它返回的是一个 Promise 数组，then 函数的参数本应是一个参数为 Promise resolves（数组）的函数，在这里使用了 axios.spread 方法。所以 axios.spread 方法是接收一个函数，返回一个新的函数，新函数的结构满足 then 函数的参数结构
  axios.spread = function spread(callback) {
      return function wrap(arr) {
          return callback.apply(null, arr);
      };
  };
  axios.Axios = Axios;

  return axios;

}));
//# sourceMappingURL=ts-axios.umd.js.map
